@@ -1,13 +1,16 @@
 from hashlib import md5
 
-from django.core.exceptions import ObjectDoesNotExist
+from django.core.exceptions import ObjectDoesNotExist, ValidationError
 from django.db import models
-from django.utils.translation import ugettext_lazy as _
+from django.db.models import Q
+from django.utils.translation import gettext_lazy as _
+from swapper import get_model_name
 
 from openwisp_users.mixins import OrgMixin
 from openwisp_utils.base import KeyField
 
 from .. import settings as app_settings
+from ..signals import device_group_changed, device_name_changed, management_ip_changed
 from ..validators import device_name_validator, mac_address_validator
 from .base import BaseModel
 
@@ -19,6 +22,8 @@ class AbstractDevice(OrgMixin, BaseModel):
     physical properties of a network device
     """
 
+    _changed_checked_fields = ['name', 'group_id', 'management_ip']
+
     name = models.CharField(
         max_length=64,
         unique=False,
@@ -61,6 +66,13 @@ class AbstractDevice(OrgMixin, BaseModel):
         help_text=_('system on chip or CPU info'),
     )
     notes = models.TextField(blank=True, help_text=_('internal notes'))
+    group = models.ForeignKey(
+        get_model_name('config', 'DeviceGroup'),
+        verbose_name=_('group'),
+        on_delete=models.SET_NULL,
+        blank=True,
+        null=True,
+    )
     # these fields are filled automatically
     # with data received from devices
     last_ip = models.GenericIPAddressField(
@@ -82,7 +94,6 @@ class AbstractDevice(OrgMixin, BaseModel):
 
     class Meta:
         unique_together = (
-            ('name', 'organization'),
             ('mac_address', 'organization'),
             ('hardware_id', 'organization'),
         )
@@ -90,6 +101,17 @@ class AbstractDevice(OrgMixin, BaseModel):
         verbose_name = app_settings.DEVICE_VERBOSE_NAME[0]
         verbose_name_plural = app_settings.DEVICE_VERBOSE_NAME[1]
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._set_initial_values_for_changed_checked_fields()
+
+    def _set_initial_values_for_changed_checked_fields(self):
+        for field in self._changed_checked_fields:
+            if self._is_deferred(field):
+                setattr(self, f'_initial_{field}', models.DEFERRED)
+            else:
+                setattr(self, f'_initial_{field}', getattr(self, field))
+
     def __str__(self):
         return (
             self.hardware_id
@@ -97,18 +119,6 @@ class AbstractDevice(OrgMixin, BaseModel):
             else self.name
         )
 
-    def clean(self):
-        """
-        modifies related config status if name
-        attribute is changed (queries the database)
-        """
-        super().clean()
-        if self._state.adding:
-            return
-        current = self.__class__.objects.get(pk=self.pk)
-        if self.name != current.name and self._has_config():
-            self.config.set_status_modified()
-
     def _has_config(self):
         return hasattr(self, 'config')
 
@@ -148,6 +158,25 @@ class AbstractDevice(OrgMixin, BaseModel):
         else:
             return KeyField.default_callable()
 
+    def _validate_unique_name(self):
+        if app_settings.DEVICE_NAME_UNIQUE:
+            if (
+                hasattr(self, 'organization')
+                and self._meta.model.objects.filter(
+                    ~Q(id=self.id),
+                    organization=self.organization,
+                    name__iexact=self.name,
+                ).exists()
+            ):
+                raise ValidationError(
+                    _('Device with this Name and Organization already exists.')
+                )
+
+    def clean(self, *args, **kwargs):
+        super().clean(*args, **kwargs)
+        self._validate_unique_name()
+        self._validate_org_relation('group', field_error='group')
+
     def save(self, *args, **kwargs):
         if not self.key:
             try:
@@ -159,6 +188,79 @@ class AbstractDevice(OrgMixin, BaseModel):
             else:
                 self.key = self.generate_key(shared_secret)
         super().save(*args, **kwargs)
+        self._check_changed_fields()
+
+    def _check_changed_fields(self):
+        self._get_initial_values_for_checked_fields()
+        # Execute method for checked for each field in self._changed_checked_fields
+        for field in self._changed_checked_fields:
+            getattr(self, f'_check_{field}_changed')()
+
+    def _is_deferred(self, field):
+        """
+        Return a boolean whether the field is deferred.
+        """
+        return field in self.get_deferred_fields()
+
+    def _get_initial_values_for_checked_fields(self):
+        # Refresh values from database only when the checked field
+        # was initially deferred, but is no longer deferred now.
+        # Store the present value of such fields because they will
+        # be overwritten fetching values from database
+        # NOTE: Initial value of a field will only remain deferred
+        # if the current value of the field is still deferred. This
+        present_values = dict()
+        for field in self._changed_checked_fields:
+            if getattr(
+                self, f'_initial_{field}'
+            ) == models.DEFERRED and not self._is_deferred(field):
+                present_values[field] = getattr(self, field)
+        # Skip fetching values from database if all of the checked fields are
+        # still deferred, or were not deferred from the begining.
+        if not present_values:
+            return
+        self.refresh_from_db(fields=present_values.keys())
+        for field in self._changed_checked_fields:
+            setattr(self, f'_initial_{field}', field)
+            setattr(self, field, present_values[field])
+
+    def _check_name_changed(self):
+        if self._initial_name == models.DEFERRED:
+            return
+
+        if self._initial_name != self.name:
+            device_name_changed.send(
+                sender=self.__class__,
+                instance=self,
+            )
+
+            if self._has_config():
+                self.config.set_status_modified()
+
+    def _check_group_id_changed(self):
+        if self._initial_group_id == models.DEFERRED:
+            return
+
+        if self._initial_group_id != self.group_id:
+            device_group_changed.send(
+                sender=self.__class__,
+                instance=self,
+                group_id=self.group_id,
+                old_group_id=self._initial_group_id,
+            )
+
+    def _check_management_ip_changed(self):
+        if self._initial_management_ip == models.DEFERRED:
+            return
+        if self.management_ip != self._initial_management_ip:
+            management_ip_changed.send(
+                sender=self.__class__,
+                management_ip=self.management_ip,
+                old_management_ip=self._initial_management_ip,
+                instance=self,
+            )
+
+        self._initial_management_ip = self.management_ip
 
     @property
     def backend(self):
