@@ -2,15 +2,19 @@ import json
 import os
 from unittest.mock import patch
 
+from django.contrib.admin.models import LogEntry
 from django.contrib.auth import get_user_model
+from django.core.exceptions import ValidationError
 from django.test import TestCase
 from django.urls import reverse
 from swapper import load_model
 
 from openwisp_users.tests.utils import TestOrganizationMixin
 
+from ...geo.tests.utils import TestGeoMixin
 from ...tests.utils import TestAdminMixin
-from .utils import CreateConfigTemplateMixin, TestVpnX509Mixin
+from .. import settings as app_settings
+from .utils import CreateConfigTemplateMixin, CreateDeviceGroupMixin, TestVpnX509Mixin
 
 devnull = open(os.devnull, 'w')
 Config = load_model('config', 'Config')
@@ -20,9 +24,13 @@ Vpn = load_model('config', 'Vpn')
 Ca = load_model('django_x509', 'Ca')
 Cert = load_model('django_x509', 'Cert')
 User = get_user_model()
+Location = load_model('geo', 'Location')
+DeviceLocation = load_model('geo', 'DeviceLocation')
+Group = load_model('openwisp_users', 'Group')
 
 
 class TestAdmin(
+    TestGeoMixin,
     CreateConfigTemplateMixin,
     TestVpnX509Mixin,
     TestAdminMixin,
@@ -35,13 +43,10 @@ class TestAdmin(
 
     app_label = 'config'
     fixtures = ['test_templates']
+    location_model = Location
+    object_model = Device
+    object_location_model = DeviceLocation
     maxDiff = None
-    operator_permission_filters = [
-        {'codename__endswith': 'config'},
-        {'codename__endswith': 'device'},
-        {'codename__endswith': 'template'},
-        {'codename__endswith': 'vpn'},
-    ]
     _device_params = {
         'name': 'test-device',
         'hardware_id': '1234',
@@ -65,6 +70,10 @@ class TestAdmin(
         'deviceconnection_set-INITIAL_FORMS': 0,
         'deviceconnection_set-MIN_NUM_FORMS': 0,
         'deviceconnection_set-MAX_NUM_FORMS': 1000,
+        'command_set-TOTAL_FORMS': 0,
+        'command_set-INITIAL_FORMS': 0,
+        'command_set-MIN_NUM_FORMS': 0,
+        'command_set-MAX_NUM_FORMS': 1000,
     }
     # WARNING - WATCHOUT
     # this class attribute is changed dynamically
@@ -183,6 +192,7 @@ class TestAdmin(
         org2 = self._create_org(name='test2org')
         inactive = self._create_org(name='inactive-org', is_active=False)
         operator = self._create_operator(organizations=[org1, inactive])
+        administrator = self._create_administrator(organizations=[org1, inactive])
         t1 = self._create_template(name='template1org', organization=org1)
         t2 = self._create_template(name='template2org', organization=org2)
         t3 = self._create_template(name='t3-inactive', organization=inactive)
@@ -215,6 +225,7 @@ class TestAdmin(
             org2=org2,
             inactive=inactive,
             operator=operator,
+            administrator=administrator,
         )
         if vpn:
             v1 = self._create_vpn(name='vpn1org', organization=org1)
@@ -302,6 +313,7 @@ class TestAdmin(
             visible=[data['org1'].name],
             hidden=[data['org2'].name, data['inactive']],
             select_widget=True,
+            administrator=True,
         )
 
     def test_vpn_ca_fk_queryset(self):
@@ -311,6 +323,7 @@ class TestAdmin(
             visible=[data['vpn1'].ca.name, data['vpn_shared'].ca.name],
             hidden=[data['vpn2'].ca.name, data['vpn_inactive'].ca.name],
             select_widget=True,
+            administrator=True,
         )
 
     def test_vpn_cert_fk_queryset(self):
@@ -320,6 +333,7 @@ class TestAdmin(
             visible=[data['vpn1'].cert.name, data['vpn_shared'].cert.name],
             hidden=[data['vpn2'].cert.name, data['vpn_inactive'].cert.name],
             select_widget=True,
+            administrator=True,
         )
 
     def test_changelist_recover_deleted_button(self):
@@ -357,6 +371,56 @@ class TestAdmin(
         response = self.client.get(path)
         self.assertNotContains(response, '// enable default templates')
 
+    def test_configuration_templates_removed(self):
+        def _update_template(templates):
+            params.update(
+                {
+                    'config-0-templates': ','.join(
+                        [str(template.pk) for template in templates]
+                    )
+                }
+            )
+            response = self.client.post(path, data=params, follow=True)
+            self.assertEqual(response.status_code, 200)
+            for template in templates:
+                self.assertContains(
+                    response, f'class="sortedm2m" checked> {template.name}'
+                )
+            return response
+
+        template = self._create_template()
+
+        # Add a new device
+        path = reverse(f'admin:{self.app_label}_device_add')
+        params = self._get_device_params(org=self._get_org())
+        response = self.client.post(path, data=params, follow=True)
+        self.assertEqual(response.status_code, 200)
+
+        config = Device.objects.get(name=params['name']).config
+        path = reverse(f'admin:{self.app_label}_device_change', args=[config.device_id])
+        params.update(
+            {
+                'config-0-id': str(config.pk),
+                'config-0-device': str(config.device_id),
+                'config-INITIAL_FORMS': 1,
+                '_continue': True,
+            }
+        )
+
+        # Add template to the device
+        _update_template(templates=[template])
+        config.refresh_from_db()
+        self.assertEqual(config.templates.count(), 1)
+        self.assertEqual(config.status, 'modified')
+        config.set_status_applied()
+        self.assertEqual(config.status, 'applied')
+
+        # Remove template from the device
+        _update_template(templates=[])
+        config.refresh_from_db()
+        self.assertEqual(config.templates.count(), 0)
+        self.assertEqual(config.status, 'modified')
+
     def test_vpn_not_contains_default_templates_js(self):
         vpn = self._create_vpn()
         path = reverse(f'admin:{self.app_label}_vpn_change', args=[vpn.pk])
@@ -386,6 +450,7 @@ class TestAdmin(
         self.client.force_login(self._get_admin())
         response = self.client.post(path, post_data)
         self.assertContains(response, 'Clone templates')
+        self.assertContains(response, 'Shared systemwide')
 
     def test_clone_templates_operator_1_org(self):
         path = reverse(f'admin:{self.app_label}_template_changelist')
@@ -406,6 +471,7 @@ class TestAdmin(
         self.client.force_login(operator)
         response = self.client.post(path, post_data)
         self.assertContains(response, 'Clone templates')
+        self.assertNotContains(response, 'Shared systemwide')
 
     def test_change_device_clean_templates(self):
         o = self._get_org()
@@ -424,17 +490,94 @@ class TestAdmin(
             }
         )
         # ensure it fails with error
-        response = self.client.post(path, params)
+        with patch.object(
+            Config, 'clean_templates', side_effect=ValidationError('test')
+        ):
+            response = self.client.post(path, params)
         self.assertContains(response, 'errors field-templates')
         # remove conflicting template and ensure doesn't error
         params['config-0-templates'] = ''
         response = self.client.post(path, params)
         self.assertNotContains(response, 'errors field-templates', status_code=302)
 
+    def test_change_device_required_template(self):
+        o = self._get_org()
+        t = Template.objects.first()
+        t.name = 'Empty'
+        t.config = {'interfaces': []}
+        t.required = True
+        t.full_clean()
+        t.save()
+        d = self._create_device(organization=o)
+        c = self._create_config(device=d, backend=t.backend, config=t.config)
+        path = reverse(f'admin:{self.app_label}_device_change', args=[d.pk])
+
+        with self.subTest(
+            'POST request without a required template is ineffective in removing it'
+        ):
+            t2 = Template.objects.get(name='radio0')
+            params = self._get_device_params(org=o)
+            params.update(
+                {
+                    'name': 'test-device-changed',
+                    'config-0-id': str(c.pk),
+                    'config-0-device': str(d.pk),
+                    'config-0-templates': str(t2.pk),
+                    'config-INITIAL_FORMS': 1,
+                }
+            )
+            response = self.client.post(path, params)
+            self.assertNotContains(response, 'errors', status_code=302)
+            c.refresh_from_db()
+            self.assertEqual(c.name, 'test-device-changed')
+            self.assertTrue(c.templates.filter(pk=t.pk).exists())
+
+        with self.subTest(
+            'Clearing all templates is ineffective in removing required template'
+        ):
+            params = self._get_device_params(org=o)
+            params.update(
+                {
+                    'name': 'test-device-templates-cleared',
+                    'config-0-id': str(c.pk),
+                    'config-0-device': str(d.pk),
+                    'config-0-templates': '',
+                    'config-INITIAL_FORMS': 1,
+                }
+            )
+            response = self.client.post(path, params)
+            self.assertNotContains(response, 'errors', status_code=302)
+            c.refresh_from_db()
+            self.assertEqual(c.name, 'test-device-templates-cleared')
+            self.assertTrue(c.templates.filter(pk=t.pk).exists())
+
+    def test_change_device_org_required_templates(self):
+        org1 = self._create_org(name='org1')
+        org2 = self._create_org(name='org2')
+        template = self._create_template(organization=org1, config={'interfaces': []})
+        device = self._create_device(organization=org1)
+        config = self._create_config(device=device)
+        path = reverse(f'admin:{self.app_label}_device_change', args=[device.pk])
+        params = self._get_device_params(org=org1)
+        params.update(
+            {
+                'name': 'test-device-changed',
+                'config-0-id': str(config.pk),
+                'config-0-device': str(device.pk),
+                'config-0-templates': str(template.pk),
+                'config-INITIAL_FORMS': 1,
+                'organization': str(org2.pk),
+            }
+        )
+        response = self.client.post(path, params, follow=True)
+        self.assertEqual(response.status_code, 200)
+        config.refresh_from_db()
+        self.assertEqual(config.templates.count(), 0)
+
     def test_download_device_config(self):
         d = self._create_device(name='download')
         self._create_config(device=d)
-        path = reverse(f'admin:{self.app_label}_device_download', args=[d.pk])
+        path = reverse(f'admin:{self.app_label}_device_download', args=[d.pk.hex])
         response = self.client.get(path)
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.get('content-type'), 'application/octet-stream')
@@ -585,10 +728,17 @@ class TestAdmin(
 
     def test_preview_device_showerror(self):
         t1 = Template.objects.get(name='dhcp')
-        t2 = Template(name='t', config=t1.config, backend='netjsonconfig.OpenWrt')
-        t2.full_clean()
-        t2.save()
-        templates = [t1, t2]
+        t2 = Template(name='t2', config=t1.config, backend=t1.backend)
+        t3 = Template(name='t3', config=t1.config, backend=t1.backend)
+        t4 = Template(
+            name='t4',
+            config={"interfaces": [{"name": "eth0", "type": "bridge", "stp": "WRONG"}]},
+            backend='netjsonconfig.OpenWrt',
+        )
+        # skip validating config to raise error later
+        t4.save()
+        # adding multiple templates to ensure the order is retained correctly
+        templates = [t1, t2, t3, t4]
         path = reverse(f'admin:{self.app_label}_device_preview')
         data = {
             'name': 'test-device',
@@ -599,7 +749,7 @@ class TestAdmin(
             'csrfmiddlewaretoken': 'test',
         }
         response = self.client.post(path, data)
-        # expect duplicate error
+        # expect error
         self.assertContains(response, '<pre class="djnjc-preformatted error')
 
     @patch('sys.stdout', devnull)
@@ -643,7 +793,10 @@ class TestAdmin(
         self.assertEqual(response.status_code, 404)
 
     def test_change_device_malformed_uuid(self):
-        path = reverse(f'admin:{self.app_label}_device_change', args=['wrong'])
+        path = reverse(
+            f'admin:{self.app_label}_device_change',
+            args=['00000000-0000-0000-0000-000000000000'],
+        )
         response = self.client.get(path)
         self.assertEqual(response.status_code, 404)
 
@@ -692,6 +845,18 @@ class TestAdmin(
         response = self.client.get(path, {'q': 'ZERO-RESULTS-PLEASE'})
         self.assertNotContains(response, 'admin-search-test')
 
+        with self.subTest('test device location search'):
+            response = self.client.get(path, {'q': 'Estonia'})
+            self.assertNotContains(response, 'admin-search-test')
+            location = self._create_location(
+                name='OW2',
+                address='Sepapaja 35, Tallinn, Estonia',
+                organization=self._get_org(),
+            )
+            self._create_object_location(content_object=d, location=location)
+            response = self.client.get(path, {'q': 'Estonia'})
+            self.assertContains(response, 'admin-search-test')
+
     def test_default_template_backend(self):
         path = reverse(f'admin:{self.app_label}_template_add')
         response = self.client.get(path)
@@ -795,6 +960,83 @@ class TestAdmin(
             response, 'value="openwisp_controller.vpn_backends.OpenVpn" selected'
         )
 
+    def test_vpn_clients_deleted(self):
+        def _update_template(templates):
+            params.update(
+                {
+                    'config-0-templates': ','.join(
+                        [str(template.pk) for template in templates]
+                    )
+                }
+            )
+            response = self.client.post(path, data=params, follow=True)
+            self.assertEqual(response.status_code, 200)
+            for template in templates:
+                self.assertContains(
+                    response, f'class="sortedm2m" checked> {template.name}'
+                )
+            return response
+
+        vpn = self._create_vpn()
+        template = self._create_template()
+        vpn_template = self._create_template(
+            name='vpn-test',
+            type='vpn',
+            vpn=vpn,
+            auto_cert=True,
+        )
+        cert_query = Cert.objects.exclude(pk=vpn.cert_id)
+
+        # Add a new device
+        path = reverse(f'admin:{self.app_label}_device_add')
+        params = self._get_device_params(org=self._get_org())
+        response = self.client.post(path, data=params, follow=True)
+        self.assertEqual(response.status_code, 200)
+
+        config = Device.objects.get(name=params['name']).config
+        self.assertEqual(config.vpnclient_set.count(), 0)
+        self.assertEqual(config.templates.count(), 0)
+
+        path = reverse(f'admin:{self.app_label}_device_change', args=[config.device_id])
+        params.update(
+            {
+                'config-0-id': str(config.pk),
+                'config-0-device': str(config.device_id),
+                'config-INITIAL_FORMS': 1,
+                '_continue': True,
+            }
+        )
+
+        with self.subTest('Adding only VpnClient template'):
+            # Adding VpnClient template to the device
+            _update_template(templates=[vpn_template])
+
+            self.assertEqual(config.templates.count(), 1)
+            self.assertEqual(config.vpnclient_set.count(), 1)
+            self.assertEqual(cert_query.count(), 1)
+
+            # Remove VpnClient template from the device
+            _update_template(templates=[])
+
+            self.assertEqual(config.templates.count(), 0)
+            self.assertEqual(config.vpnclient_set.count(), 0)
+            self.assertEqual(cert_query.count(), 0)
+
+        with self.subTest('Add VpnClient template along with another template'):
+            # Adding templates to the device
+            _update_template(templates=[template, vpn_template])
+
+            self.assertEqual(config.templates.count(), 2)
+            self.assertEqual(config.vpnclient_set.count(), 1)
+            self.assertEqual(cert_query.count(), 1)
+
+            # Remove VpnClient template from the device
+            _update_template(templates=[template])
+
+            self.assertEqual(config.templates.count(), 1)
+            self.assertEqual(config.vpnclient_set.count(), 0)
+            self.assertEqual(cert_query.count(), 0)
+
     def test_ip_not_in_add_device(self):
         path = reverse(f'admin:{self.app_label}_device_add')
         response = self.client.get(path)
@@ -915,13 +1157,19 @@ class TestAdmin(
         self.assertContains(response, '{} (Clone 2)'.format(t.name))
         response = self.client.post(path, data, follow=True)
         self.assertContains(response, '{} (Clone 3)'.format(t.name))
+        path = reverse('admin:index')
+        response = self.client.get(path)
+        self.assertIn('test-template (Clone 3)', str(response.content))
+        self.assertIn('test-template (Clone 2)', str(response.content))
+        self.assertIn('test-template (Clone)', str(response.content))
+        self.assertEqual(LogEntry.objects.all().count(), 3)
 
     def test_get_template_default_values(self):
         t1 = self._create_template(name='t1', default_values={'name1': 'test1'})
         path = reverse('admin:get_template_default_values')
 
         with self.subTest('get default values for one template'):
-            with self.assertNumQueries(1):
+            with self.assertNumQueries(3):
                 r = self.client.get(path, {'pks': f'{t1.pk}'})
                 self.assertEqual(r.status_code, 200)
                 expected = {'default_values': {'name1': 'test1'}}
@@ -929,7 +1177,7 @@ class TestAdmin(
 
         with self.subTest('get default values for multiple templates'):
             t2 = self._create_template(name='t2', default_values={'name2': 'test2'})
-            with self.assertNumQueries(1):
+            with self.assertNumQueries(3):
                 r = self.client.get(path, {'pks': f'{t1.pk},{t2.pk}'})
                 self.assertEqual(r.status_code, 200)
                 expected = {'default_values': {'name1': 'test1', 'name2': 'test2'}}
@@ -986,7 +1234,82 @@ class TestAdmin(
             path = reverse(f'admin:{self.app_label}_device_change', args=[d.pk])
             self._test_system_context_field_helper(path)
 
+    def test_no_system_context(self):
+        self._create_template()
+        old_context = app_settings.CONTEXT.copy()
+        app_settings.CONTEXT = {}
+        path = reverse(f'admin:{self.app_label}_template_add')
+        r = self.client.get(path)
+        self.assertContains(
+            r, 'There are no system defined variables available right now'
+        )
+        app_settings.CONTEXT = old_context
+
     @classmethod
     def tearDownClass(cls):
         super().tearDownClass()
         devnull.close()
+
+
+class TestDeviceGroupAdmin(
+    CreateDeviceGroupMixin, TestOrganizationMixin, TestAdminMixin, TestCase
+):
+    app_label = 'config'
+
+    def setUp(self):
+        self.client.force_login(self._get_admin())
+
+    def test_multitenant_admin(self):
+        org1 = self._create_org(name='org1')
+        org2 = self._create_org(name='org2')
+        user = self._create_org_user(
+            organization=org1, is_admin=True, user=self._get_operator()
+        ).user
+        user.groups.add(Group.objects.get(name='Operator'))
+
+        self._create_device_group(name='Org1 APs', organization=org1)
+        self._create_device_group(name='Org2 APs', organization=org2)
+        self.client.logout()
+        self.client.force_login(user)
+
+        response = self.client.get(
+            reverse(f'admin:{self.app_label}_devicegroup_changelist')
+        )
+        self.assertContains(response, 'Org1 APs')
+        self.assertNotContains(response, 'Org2 APs')
+
+    def test_organization_filter(self):
+        org1 = self._create_org(name='org1')
+        org2 = self._create_org(name='org1')
+        self._create_device_group(name='Org1 APs', organization=org1)
+        self._create_device_group(name='Org2 APs', organization=org2)
+        url = reverse(f'admin:{self.app_label}_devicegroup_changelist')
+        query = f'?organization__id__exact={org1.pk}'
+        response = self.client.get(url)
+        self.assertContains(response, 'Org1 APs')
+        self.assertContains(response, 'Org2 APs')
+
+        response = self.client.get(f'{url}{query}')
+        self.assertContains(response, 'Org1 APs')
+        self.assertNotContains(response, 'Org2 APs')
+
+    def test_admin_menu_groups(self):
+        # Test menu group (openwisp-utils menu group) for Device Group, Template
+        # and Vpn models
+        models = ['devicegroup', 'template', 'vpn']
+        self.client.force_login(self._get_admin())
+        response = self.client.get(reverse('admin:index'))
+        for model in models:
+            with self.subTest(f'test menu group link for {model} model'):
+                url = reverse(f'admin:{self.app_label}_{model}_changelist')
+                self.assertContains(response, f' class="mg-link" href="{url}"')
+        with self.subTest('test "Configurations" group is registered'):
+            self.assertContains(
+                response,
+                '<div class="mg-dropdown-label">Configurations </div>',
+                html=True,
+            )
+        # Test Device is registered as menu item
+        with self.subTest('test device menu item is registered'):
+            url = reverse(f'admin:{self.app_label}_device_changelist')
+            self.assertContains(response, f' class="menu-item" href="{url}"')
