@@ -1,9 +1,9 @@
+'use strict';
 django.jQuery(function ($) {
-    'use strict';
     var showField = function () {
         $('.form-row.field-organization').show();
     };
-    $('.field-operation_type select').on('change', function (e) {
+    $('.field-operation_type select').on('change', function () {
         showField();
     });
 });
